@@ -5,10 +5,13 @@ import os
 import ast
 import unittest
 from unittest import TestCase # to allow from pyt import TestCase, Assert
+#from unittest import TextTestRunner, TextTestResult
 import sys
 import inspect
 import imp
 import importlib
+from contextlib import contextmanager
+from StringIO import StringIO
 
 from . import echo
 
@@ -311,7 +314,6 @@ class TestLoader(unittest.TestLoader):
             ti.raise_any_error()
 
         return ts
-        #return super(TestLoader, self).loadTestsFromName(*args, **kwargs)
 
     def loadTestsFromNames(self, names, *args, **kwargs):
         ts = self.suiteClass()
@@ -320,7 +322,42 @@ class TestLoader(unittest.TestLoader):
             ts.addTest(name_suite)
 
         return ts
-        #return super(TestLoader, self).loadTestsFromNames(*args, **kwargs)
+
+
+class TestResult(unittest.TextTestResult):
+    stdout_stream = sys.stdout
+    stderr_stream = sys.stderr
+
+    stdout_buffer = StringIO()
+    stderr_buffer = StringIO()
+
+#     def startTest(self, test):
+#         #pout.v('startTest')
+#         #pout.v("before start", id(sys.stdout), id(sys.stderr))
+#         super(TestResult, self).startTest(test)
+#         #pout.v("after start", id(sys.stdout), id(sys.stderr))
+
+#     def stopTest(self, test):
+#         #pout.v('stopTest')
+#         #pout.v("buffered stop", id(sys.stdout), id(sys.stderr))
+#         super(TestResult, self).stopTest(test)
+#         #pout.v("original stop", id(sys.stdout), id(sys.stderr))
+
+    def __init__(self, *args, **kwargs):
+        super(TestResult, self).__init__(*args, **kwargs)
+        self._original_stdout = self.stdout_stream
+        self._original_stderr = self.stderr_stream
+        self._stdout_buffer = self.stdout_buffer
+        self._stderr_buffer = self.stderr_buffer
+
+
+class TestRunner(unittest.TextTestRunner):
+    resultclass = TestResult
+
+    def __init__(self, stream=None, *args, **kwargs):
+        if not stream:
+            stream = self.resultclass.stderr_stream
+        super(TestRunner, self).__init__(stream=stream, *args, **kwargs)
 
 
 def run_test(name, basedir, **kwargs):
@@ -330,15 +367,19 @@ def run_test(name, basedir, **kwargs):
     **kwargs -- dict -- all other args to pass to unittest
     '''
     ret_code = 0
+
+    if kwargs.get('buffer', False):
+        sys.stdout = TestResult.stdout_buffer
+        sys.stderr = TestResult.stderr_buffer
+
     tl = TestLoader(basedir)
 
     kwargs.setdefault('argv', ['run_test'])
     kwargs['argv'].append(name)
 
-    #kwargs.setdefault('module', tl.module)
     kwargs.setdefault('exit', False)
-    #kwargs.setdefault('failfast', True)
     kwargs.setdefault('testLoader', tl)
+    kwargs.setdefault('testRunner', TestRunner)
 
     # https://docs.python.org/2/library/unittest.html#unittest.main
     ret = unittest.main(**kwargs)
@@ -348,6 +389,10 @@ def run_test(name, basedir, **kwargs):
     elif not ret.result.testsRun:
         ret_code = 1
 
+    if kwargs.get('buffer', False):
+        sys.stdout = TestResult.stdout_stream
+        sys.stderr = TestResult.stderr_stream
+
     echo.debug('Test returned: {}', ret_code)
     return ret_code
 
